@@ -1,6 +1,6 @@
 /*****************************************************************************
- * Secure Locate v1.5                                                        *
- * May 5, 1999                                                               *
+ * Secure Locate v1.6                                                        *
+ * May 28, 1999                                                              *
  * Programmed by: Kevin Lindsay                                              *
  * Copyright (c) 1998 Kevin Lindsay & Netnation Communications Inc.          *
  *                                                                           *
@@ -22,6 +22,11 @@
  *                  is readable.                                             *
  *           R.G. Wood <rgwood@stormix.com>                                  *
  *                * Made a Debian Package for Secure Locate                  *
+ *           Alexander V. Lukyanov" <lav@long.yar.ru>                        *
+ *                * Fixed some performance issues that I over looked.  Thanx *
+ *                  To Alex, slocate -u is much faster!                      *
+ *           Matt Heckaman <matt@MLINK.NET>                                  *
+ *                * Created a patch to make Secure Locate work with FreeBSD. *
  *                                                                           *
  * Report any Bugs to: klindsay@mkintraweb.com                               *
  *                                                                           *
@@ -71,7 +76,7 @@
 
 /* GLOBALS */
 
-#define VERSION "Secure Locate v1.5 - Released May 5, 1999\n"
+#define VERSION "Secure Locate v1.6 - Released May 28, 1999\n"
 #define GRPFILE "/etc/group"
 #define SLOC_ESC 0x80
 #define SLOC_GRP "slocate"
@@ -85,9 +90,18 @@
 #define WARN_MESSAGE "8 days"
 
 char **SLOCATE_PATH = NULL;
+
+/* More fitting paths for FreeBSD -matt */
+#if defined(__FreeBSD__)
+char *SLOCATEDB = "/var/db/slocate/slocate.db";
+char *TMPSLOCATEDB = "/var/db/slocate/slocate.tmp";
+char *SLOCATEDB_DIR = "/var/db/slocate/";
+#else
+/* These paths are good for Linux machines */
 char *SLOCATEDB = "/var/lib/slocate/slocate.db";
 char *TMPSLOCATEDB = "/var/lib/slocate/slocate.tmp";
 char *SLOCATEDB_DIR = "/var/lib/slocate/";
+#endif
 char *EXCLUDE_DIR;
 int EXCLUDE=0;
 int VERBOSE=0;
@@ -97,6 +111,7 @@ char *regexp;
 char *progname;
 char *tmp_path=NULL;
 char *cur_dir=NULL;
+char prog_CWD[4096];
 short fr_num=0;
 int t_num=0;
 uid_t UID;
@@ -431,7 +446,6 @@ load_dir(int dir_fd, dir_item *listptr)
 
       while (!strcmp(filename->d_name,".") || !strcmp(filename->d_name,"..")) {
          top=1;
-         seekdir(recdir,telldir(recdir));
          if ((filename = readdir(recdir)) == NULL)
              break;
       }
@@ -477,7 +491,6 @@ load_dir(int dir_fd, dir_item *listptr)
       
       listptr->root = 0;
 
-      seekdir(recdir,telldir(recdir));      
    }
    
    closedir(recdir);
@@ -545,7 +558,7 @@ get_cur_dir(dir_item *listptr, char *dirstr)
 /* FRCODE - Incremental Encoding algorithm */
 
 int
-frcode(char *dir_path, char *filename, struct stat statres)
+frcode(FILE *fd,char *dir_path, char *filename)
 {
    int res=0;
    char *cur_path;
@@ -554,7 +567,6 @@ frcode(char *dir_path, char *filename, struct stat statres)
    char *ptr2;
 
    int i;
-   FILE *fd;   
    
    cur_path = malloc(strlen(dir_path)+strlen(filename)+1);
    cur_path[0]='\0';
@@ -596,7 +608,6 @@ frcode(char *dir_path, char *filename, struct stat statres)
    tmp_path[0] = '\0';
    strcat(tmp_path,cur_path);
    
-   fd = fopen(TMPSLOCATEDB,"a");
    if (fr_num > 127 || fr_num < -127) {
       putc (SLOC_ESC, fd);
       put_short (fr_num, fd);
@@ -606,8 +617,6 @@ frcode(char *dir_path, char *filename, struct stat statres)
    fputs(new_path,fd);
    putc('\0',fd);
 
-   fclose(fd);
-   
    if (first) {
       chmod(TMPSLOCATEDB,00640);
       chown(TMPSLOCATEDB,UID,SLOC_GID);
@@ -643,13 +652,13 @@ create_db(char *dirstr)
    
    fd = fopen(TMPSLOCATEDB,"w");
    putc(slevel,fd);
-   fclose(fd);
    
    if (dirstr) {
       /* Make sure there are no problems accessing the source directory */
       if (lstat(dirstr,&statres) == -1) {
          if (!QUIET)
              fprintf(stderr,"%s: lstat: d%s/: %s\n",progname,dirstr,strerror(errno));
+    fclose(fd);
          return(-1);
       }
    }
@@ -680,12 +689,14 @@ create_db(char *dirstr)
    if ((dir_fd = open(dirstr,O_RDONLY)) == -1) {
       if (!QUIET)
           fprintf(stderr,"%s: open: %s: %s\n",progname,dirstr,strerror(errno));
+      fclose(fd);
       return(-1);
    }
    
    /* Change CWD to source dir */
    if (!MYfchdir(dir_fd,dirstr,"CDB_1")) {
       close(dir_fd);
+      fclose(fd);
       return(0);
    }
    
@@ -710,10 +721,8 @@ create_db(char *dirstr)
          }
          list_down=0;         
          while (listptr != NULL) {
-            lstat(listptr->name,&statres);
             if (!EXCLUDE || (EXCLUDE && !match_exclude(get_cur_dir(listptr,dirstr),listptr->name))) {               
-               if (statres.st_uid < 65535)
-                   frcode(get_cur_dir(listptr,dirstr),listptr->name,statres);               
+               frcode(fd,get_cur_dir(listptr,dirstr),listptr->name);               
             }
             if (!listptr->type) {
                listptr=listptr->down;
@@ -725,6 +734,7 @@ create_db(char *dirstr)
       } else {
          if (!MYchdir("..","create_db")) {
             close(dir_fd);
+       fclose(fd);
             return(1);
          }         
          listptr=listptr->left;
@@ -748,6 +758,7 @@ create_db(char *dirstr)
             
             if (!MYchdir("..","CDB_2")) {
                close(dir_fd);
+          fclose(fd);
                return(1);
             }
             
@@ -756,6 +767,7 @@ create_db(char *dirstr)
             if ((dir_fd = open(".",O_RDONLY)) == -1) {
                if (!QUIET)
                    fprintf(stderr,"%s: open: CDB_3: %s\n",progname,strerror(errno));
+          fclose(fd);
                return(1);
             }
             
@@ -791,6 +803,8 @@ create_db(char *dirstr)
 
    }   
    
+   fclose(fd);
+   chdir(prog_CWD);
    rename(TMPSLOCATEDB,SLOCATEDB);
    chmod(SLOCATEDB,00640);
    chown(SLOCATEDB,UID,SLOC_GID);   
@@ -964,11 +978,21 @@ main(int args, char **argv)
    int i=0;
 
 
-   progname = ((p = strrchr(argv[0],'/')) ? p+1 : *argv);   
+   progname = ((p = strrchr(argv[0],'/')) ? p+1 : *argv);     
    
    if (!strcmp(progname,"updatedb"))
      ROOTDIR=1;
    
+   /* Get Current Working Directory */
+   
+   getcwd(prog_CWD, 4095);
+   
+   if (strlen(prog_CWD) == 4095) {
+      fprintf(stderr,"%s: Current Working Directory is too large!",progname);
+      exit(1);
+   }
+
+   
    if (args < 2 && !ROOTDIR)
      usage();
    
